@@ -11,9 +11,16 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: bi_funct.c,v 1.127 2024/01/23 21:22:25 tom Exp $
+ * $MawkId: bi_funct.c,v 1.140 2024/12/14 12:53:14 tom Exp $
  */
 
+#define Visible_ARRAY
+#define Visible_BI_REC
+#define Visible_CELL
+#define Visible_RE_DATA
+#define Visible_STRING
+#define Visible_SYMTAB
+
 #include <mawk.h>
 #include <bi_funct.h>
 #include <bi_vars.h>
@@ -23,7 +30,6 @@ the GNU General Public License, version 2, 1991.
 #include <fin.h>
 #include <field.h>
 #include <regexp.h>
-#include <repl.h>
 
 #include <ctype.h>
 #include <math.h>
@@ -37,6 +43,12 @@ the GNU General Public License, version 2, 1991.
 #include <bsd/stdlib.h>		/* prototype arc4random */
 #endif
 
+#if defined(HAVE_GETTIMEOFDAY) && defined(HAVE_SYS_TIME_H)
+#include <sys/time.h>
+#else
+#undef HAVE_GETTIMEOFDAY
+#endif
+
 #if defined(WINVER) && (WINVER >= 0x501)
 #include <windows.h>
 #endif
@@ -56,7 +68,7 @@ const BI_REC bi_funct[] =
 
    { "index",    bi_index,    2, 2 },
    { "substr",   bi_substr,   2, 3 },
-   { "sprintf",  bi_sprintf,  1, 255 },
+   { "sprintf",  bi_sprintf,  1, MAX_ARGS },
    { "sin",      bi_sin,      1, 1 },
    { "cos",      bi_cos,      1, 1 },
    { "atan2",    bi_atan2,    2, 2 },
@@ -81,7 +93,7 @@ const BI_REC bi_funct[] =
    { "strftime", bi_strftime, 0, 3 },
 #endif
 
-   { (char *)    0, (PF_CP) 0, 0, 0 }
+   { "",         (PF_CP) 0, 0, 0 }
 };
 /* *INDENT-ON* */
 
@@ -92,7 +104,7 @@ bi_funct_init(void)
     register const BI_REC *p;
     register SYMTAB *stp;
 
-    for (p = bi_funct; p->name; p++) {
+    for (p = bi_funct; p->name[0]; p++) {
 	stp = insert(p->name);
 	stp->type = ST_BUILTIN;
 	stp->stval.bip = p;
@@ -120,7 +132,7 @@ bi_length(CELL *sp)
 {
     size_t len;
 
-    TRACE_FUNC("bi_length", sp);
+    TRACE_FUNC2("bi_length", sp, 1);
 
     if (sp->type < C_STRING)
 	cast1_to_s(sp);
@@ -137,7 +149,7 @@ bi_length(CELL *sp)
 CELL *
 bi_alength(CELL *sp)
 {
-    TRACE_FUNC("bi_alength", sp);
+    TRACE_FUNC2("bi_alength", sp, 1);
 
     sp->type = C_DOUBLE;
     sp->dval = (double) ((ARRAY) sp->ptr)->size;
@@ -151,7 +163,7 @@ str_str(char *target, size_t target_len, const char *key, size_t key_len)
     register int k = key[0];
     int k1;
     const char *prior;
-    char *result = 0;
+    char *result = NULL;
 
     switch (key_len) {
     case 0:
@@ -195,7 +207,7 @@ bi_index(CELL *sp)
     size_t idx;
     size_t len;
 
-    TRACE_FUNC("bi_index", sp);
+    TRACE_FUNC2("bi_index", sp, 2);
 
     sp--;
     if (TEST2(sp) != TWO_STRINGS)
@@ -309,7 +321,7 @@ bi_match(CELL *sp)
     char *p;
     size_t length;
 
-    TRACE_FUNC("bi_match", sp);
+    TRACE_FUNC2("bi_match", sp + 1, 2);
 
     if (sp->type != C_RE)
 	cast_to_RE(sp);
@@ -351,7 +363,7 @@ bi_to##case(CELL *sp) \
     size_t len; \
     register char *p, *q; \
 \
-    TRACE_FUNC("bi_to" #case, sp); \
+    TRACE_FUNC2("bi_to" #case, sp, 1); \
 \
     if (sp->type != C_STRING) \
         cast1_to_s(sp); \
@@ -381,7 +393,7 @@ bi_systime(CELL *sp)
     time_t result;
     time(&result);
 
-    TRACE_FUNC("bi_systime", sp);
+    TRACE_FUNC2("bi_systime", sp, 0);
 
     sp++;
     sp->type = C_DOUBLE;
@@ -402,7 +414,7 @@ bi_mktime(CELL *sp)
     struct tm my_tm;
     STRING *sval = string(sp);
 
-    TRACE_FUNC("bi_mktime", sp);
+    TRACE_FUNC2("bi_mktime", sp, 1);
 
     if (!sval)
 	goto error;
@@ -457,7 +469,7 @@ bi_strftime(CELL *sp)
     struct tm *ptm;
     int n_args;
     int utc;
-    STRING *sval = 0;		/* strftime(sval->str, timestamp, utc) */
+    STRING *sval = NULL;		/* strftime(sval->str, timestamp, utc) */
     size_t result;
 
     TRACE_FUNC("bi_strftime", sp);
@@ -545,7 +557,7 @@ fplib_err(
 CELL *
 bi_sin(CELL *sp)
 {
-    TRACE_FUNC("bi_sin", sp);
+    TRACE_FUNC2("bi_sin", sp, 1);
 
 #if ! STDC_MATHERR
     if (sp->type != C_DOUBLE)
@@ -570,7 +582,7 @@ bi_sin(CELL *sp)
 CELL *
 bi_cos(CELL *sp)
 {
-    TRACE_FUNC("bi_cos", sp);
+    TRACE_FUNC2("bi_cos", sp, 1);
 
 #if ! STDC_MATHERR
     if (sp->type != C_DOUBLE)
@@ -595,7 +607,7 @@ bi_cos(CELL *sp)
 CELL *
 bi_atan2(CELL *sp)
 {
-    TRACE_FUNC("bi_atan2", sp);
+    TRACE_FUNC2("bi_atan2", sp, 2);
 
 #if  !	STDC_MATHERR
     sp--;
@@ -619,7 +631,7 @@ bi_atan2(CELL *sp)
 CELL *
 bi_log(CELL *sp)
 {
-    TRACE_FUNC("bi_log", sp);
+    TRACE_FUNC2("bi_log", sp, 1);
 
 #if ! STDC_MATHERR
     if (sp->type != C_DOUBLE)
@@ -644,7 +656,7 @@ bi_log(CELL *sp)
 CELL *
 bi_exp(CELL *sp)
 {
-    TRACE_FUNC("bi_exp", sp);
+    TRACE_FUNC2("bi_exp", sp, 1);
 
 #if  ! STDC_MATHERR
     if (sp->type != C_DOUBLE)
@@ -670,7 +682,7 @@ bi_exp(CELL *sp)
 CELL *
 bi_int(CELL *sp)
 {
-    TRACE_FUNC("bi_int", sp);
+    TRACE_FUNC2("bi_int", sp, 1);
 
     if (sp->type != C_DOUBLE)
 	cast1_to_d(sp);
@@ -681,7 +693,7 @@ bi_int(CELL *sp)
 CELL *
 bi_sqrt(CELL *sp)
 {
-    TRACE_FUNC("bi_sqrt", sp);
+    TRACE_FUNC2("bi_sqrt", sp, 1);
 
 #if  ! STDC_MATHERR
     if (sp->type != C_DOUBLE)
@@ -738,13 +750,13 @@ initial_seed(void)
 #if defined(HAVE_CLOCK_GETTIME)
     struct timespec data;
     if (clock_gettime(CLOCK_REALTIME, &data) == 0)
-	result = (data.tv_sec * 1000000000L) + data.tv_nsec;
+	result = (((double) data.tv_sec * 1e9) + (double) data.tv_nsec);
     else
 	result = 0.0;
 #elif defined(HAVE_GETTIMEOFDAY)
     struct timeval data;
     if (gettimeofday(&data, (struct timezone *) 0) == 0)
-	result = (data.tv_sec * 1000000) + data.tv_usec;
+	result = (((double) data.tv_sec * 1e6) + (double) data.tv_usec);
     else
 	result = 0.0;
 #elif defined(WINVER) && (WINVER >= 0x501)
@@ -769,14 +781,14 @@ bi_srand(CELL *sp)
 #ifdef USE_SYSTEM_SRAND
     static CELL cseed =
     {
-	C_DOUBLE, 0, 0, 1.0
+	C_DOUBLE, 0, NULL, 1.0
     };
     double seed32;
 #endif
 
     CELL c;
 
-    TRACE_FUNC("bi_srand", sp);
+    TRACE_FUNC2("bi_srand", sp, (sp->type != C_NOINIT));
 
     if (sp->type == C_NOINIT)	/* seed off clock */
     {
@@ -824,7 +836,7 @@ bi_srand(CELL *sp)
 CELL *
 bi_rand(CELL *sp)
 {
-    TRACE_FUNC("bi_rand", sp);
+    TRACE_FUNC2("bi_rand", sp, 0);
 
 #ifdef USE_SYSTEM_SRAND
     {
@@ -861,7 +873,7 @@ bi_close(CELL *sp)
 {
     int x;
 
-    TRACE_FUNC("bi_close", sp);
+    TRACE_FUNC2("bi_close", sp, 1);
 
     if (sp->type < C_STRING)
 	cast1_to_s(sp);
@@ -878,9 +890,9 @@ bi_fflush(CELL *sp)
 {
     int ret = 0;
 
-    TRACE_FUNC("bi_fflush", sp);
+    TRACE_FUNC2("bi_fflush", sp, (sp->type != C_NOINIT));
 
-    if (sp->type == 0)
+    if (sp->type == C_NOINIT)
 	fflush(stdout);
     else {
 	sp--;
@@ -897,11 +909,11 @@ bi_fflush(CELL *sp)
 }
 
 CELL *
-bi_system(CELL *sp GCC_UNUSED)
+bi_system(CELL *sp)
 {
     int ret_val;
 
-    TRACE_FUNC("bi_system", sp);
+    TRACE_FUNC2("bi_system", sp + 1, 1);
 
     if (sp->type < C_STRING)
 	cast1_to_s(sp);
@@ -937,15 +949,15 @@ CELL *
 bi_getline(CELL *sp)
 {
     CELL tc;
-    CELL *cp = 0;
-    char *p = 0;
+    CELL *cp = NULL;
+    char *p = NULL;
     size_t len = 0;
     FIN *fin_p;
 
     TRACE_FUNC("bi_getline", sp);
 
     switch (sp->type) {
-    case 0:
+    case C_NOINIT:
 	sp--;
 	if (!main_fin)
 	    open_main();
@@ -1073,7 +1085,7 @@ bi_sub(CELL *sp)
 		     &middle_len,
 		     0);
 
-    if (middle != 0) {
+    if (middle != NULL) {
 	size_t front_len = (size_t) (middle - front);
 	char *back = middle + middle_len;
 	size_t back_len = string(&sc)->len - front_len - middle_len;
@@ -1161,7 +1173,7 @@ gsub3(PTR re, CELL *repl, CELL *target)
 	 * that can be an empty string, e.g., for "*" or "?".  The length
 	 * is in 'howmuch'.
 	 */
-	if (where != 0) {
+	if (where != NULL) {
 	    have = (size_t) (where - (input->str + j));
 	    if (have) {
 		skip0 = -1;
@@ -1177,11 +1189,11 @@ gsub3(PTR re, CELL *repl, CELL *target)
 	    TRACE(("\n"));
 
 	    if (repl->type == C_REPLV) {
-		if (xrepl.ptr == 0 ||
+		if (xrepl.ptr == NULL ||
 		    string(&xrepl)->len != howmuch ||
 		    (howmuch != 0 &&
 		     memcmp(string(&xrepl)->str, where, howmuch))) {
-		    if (xrepl.ptr != 0)
+		    if (xrepl.ptr != NULL)
 			repl_destroy(&xrepl);
 		    sval = new_STRING1(where, howmuch);
 		    cellcpy(&xrepl, repl);
