@@ -1,6 +1,6 @@
 /********************************************
 error.c
-copyright 2008-2020,2023 Thomas E. Dickey
+copyright 2008-2023,2024 Thomas E. Dickey
 copyright 1991-1994,1995 Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,9 +11,13 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: error.c,v 1.26 2023/08/01 21:50:59 tom Exp $
+ * $MawkId: error.c,v 1.31 2024/12/14 12:53:14 tom Exp $
  */
 
+#define Visible_CELL
+#define Visible_STRING
+#define Visible_SYMTAB
+
 #include <mawk.h>
 #include <scan.h>
 #include <bi_vars.h>
@@ -21,7 +25,7 @@ the GNU General Public License, version 2, 1991.
 #ifdef DEBUG
 #define FLUSH() fflush(stdout)
 #else
-#define FLUSH() /* nothing */
+#define FLUSH()			/* nothing */
 #endif
 
 /* for run time error messages only */
@@ -106,12 +110,15 @@ missing(int c, const char *n, unsigned ln)
 	s0 = s1 = "";
 
     errmsg(0, "%s%sline %u: missing %c near %s", s0, s1, ln, c, n);
+
+    if (++compile_error_count >= MAX_COMPILE_ERRORS)
+	mawk_exit(2);
 }
 
 void
 yyerror(const char *s GCC_UNUSED)
 {
-    const char *ss = 0;
+    const char *ss = NULL;
     const struct token_str *p;
     const int *ip;
 
@@ -130,7 +137,7 @@ yyerror(const char *s GCC_UNUSED)
 		if (*ip == current_token) {
 		    missing(')', ss, token_lineno);
 		    paren_cnt = 0;
-		    goto done;
+		    return;
 		}
 
 	if (brace_cnt)
@@ -138,17 +145,18 @@ yyerror(const char *s GCC_UNUSED)
 		if (*ip == current_token) {
 		    missing('}', ss, token_lineno);
 		    brace_cnt = 0;
-		    goto done;
+		    return;
 		}
 
 	compile_error("syntax error at or near %s", ss);
 
-    } else			/* special cases */
+    } else {			/* special cases */
 	switch (current_token) {
 	case UNEXPECTED:
 	    unexpected_char();
-	    goto done;
+	    break;
 
+#ifndef LCOV_UNUSED
 	case BAD_DECIMAL:
 	    compile_error(
 			     "syntax error in decimal constant %s",
@@ -160,16 +168,13 @@ yyerror(const char *s GCC_UNUSED)
 			     "syntax error at or near /%s/",
 			     string_buff);
 	    break;
+#endif
 
 	default:
 	    compile_error("syntax error");
 	    break;
 	}
-    return;
-
-  done:
-    if (++compile_error_count == MAX_COMPILE_ERRORS)
-	mawk_exit(2);
+    }
 }
 
 /* generic error message with a hook into the system error
@@ -185,7 +190,7 @@ errmsg(int errnum, const char *format, ...)
 
 #if OPT_TRACE > 0
     va_start(args, format);
-    Trace("\n?? errmsg \n");
+    Trace("\n?? errmsg: ");
     TraceVA(format, args);
     Trace("\n");
     va_end(args);
@@ -287,12 +292,14 @@ unexpected_char(void)
 	fprintf(stderr, "unexpected character '%c'\n", c);
     else
 	fprintf(stderr, "unexpected character 0x%02x\n", c);
+    if (++compile_error_count >= MAX_COMPILE_ERRORS)
+	mawk_exit(2);
 }
 
 const char *
 type_to_str(int type)
 {
-    const char *retval = 0;
+    const char *retval = NULL;
 
     switch (type) {
     case ST_NONE:
@@ -301,15 +308,18 @@ type_to_str(int type)
     case ST_VAR:
 	retval = "variable";
 	break;
+#ifndef LCOV_UNUSED
     case ST_KEYWORD:
 	retval = "keyword";
 	break;
     case ST_BUILTIN:
 	retval = "builtin";
 	break;
+#endif
     case ST_ARRAY:
 	retval = "array";
 	break;
+#ifndef LCOV_UNUSED
     case ST_FIELD:
 	retval = "field";
 	break;
@@ -322,6 +332,7 @@ type_to_str(int type)
     case ST_FUNCT:
 	retval = "function";
 	break;
+#endif
     case ST_LOCAL_VAR:
 	retval = "local variable";
 	break;
