@@ -1,6 +1,6 @@
 /********************************************
 execute.c
-copyright 2008-2020,2023, Thomas E. Dickey
+copyright 2008-2023,2024, Thomas E. Dickey
 copyright 1991-1995,1996, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,9 +11,18 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: execute.c,v 1.54 2023/11/27 00:48:08 tom Exp $
+ * $MawkId: execute.c,v 1.64 2024/12/14 12:53:14 tom Exp $
  */
 
+#define Visible_ARRAY
+#define Visible_BI_REC
+#define Visible_CELL
+#define Visible_DEFER_LEN
+#define Visible_FBLOCK
+#define Visible_RE_DATA
+#define Visible_STRING
+#define Visible_SYMTAB
+
 #include <mawk.h>
 #include <files.h>
 #include <code.h>
@@ -23,7 +32,6 @@ the GNU General Public License, version 2, 1991.
 #include <bi_funct.h>
 #include <bi_vars.h>
 #include <regexp.h>
-#include <repl.h>
 #include <fin.h>
 
 #include <math.h>
@@ -36,19 +44,18 @@ static char dz_msg[] = "division by zero";
 #define	 CHECK_DIVZERO(x) do { if ((x) == 0.0 ) rt_error(dz_msg); } while (0)
 #endif
 
-#define	 inc_sp()   if ( ++sp == stack_danger) eval_overflow()
-#define  dec_sp()   if ( sp-- == (stack_base - 1)) eval_underflow()
-
 #define	 SAFETY	   16
-#define	 DANGER	   (EVAL_STACK_SIZE-SAFETY)
+#define	 DANGER	   (EVAL_STACK_SIZE - SAFETY - MAX_ARGS)
 
 /*  The stack machine that executes the code */
 
 CELL eval_stack[EVAL_STACK_SIZE];
 /* these can move for deep recursion */
 static CELL *stack_base = eval_stack;
+static CELL *stack_under = eval_stack;
 static CELL *stack_danger = eval_stack + DANGER;
 
+#ifdef DEBUG
 static void
 eval_overflow(void)
 {
@@ -61,6 +68,16 @@ eval_underflow(void)
     bozo("eval stack underflow");
 }
 
+#define	 inc_sp()   if ( ++sp == stack_danger) eval_overflow()
+#define  dec_sp()   if ( sp-- == stack_under)  eval_underflow()
+
+#else
+
+#define	 inc_sp()   ++sp
+#define  dec_sp()   sp--
+
+#endif
+
 /* holds info for array loops (on a stack) */
 typedef struct aloop_state {
     struct aloop_state *link;
@@ -120,11 +137,19 @@ execute(INST * cdp,		/* code ptr, start execution here */
     ALOOP_STATE *aloop_state = (ALOOP_STATE *) 0;
 
     /* for moving the eval stack on deep recursion */
-    CELL *old_stack_base = 0;
-    CELL *old_sp = 0;
+    CELL *old_stack_base = NULL;
+    CELL *old_sp = NULL;
 
 #ifdef	DEBUG
-    CELL *entry_sp = sp;
+    CELL *entry_sp;
+#endif
+
+    if (sp == NULL) {
+	stack_under = stack_base;
+	sp = --stack_under;
+    }
+#ifdef	DEBUG
+    entry_sp = sp;
 #endif
 
     if (fp) {
@@ -149,7 +174,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	TRACE(("execute %s sp(%ld:%s)\n",
 	       da_op_name(cdp),
 	       (long) (sp - stack_base),
-	       da_type_name(sp)));
+	       (sp == stack_under) ? "?" : da_type_name(sp)));
 
 	switch ((cdp++)->op) {
 
@@ -325,7 +350,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	       has an ARRAY in the ptr field, replace expr
 	       with  array[expr]
 	     */
-	    if (fp != 0) {
+	    if (fp != NULL) {
 		cp = array_find((ARRAY) fp[(cdp++)->op].ptr, sp, CREATE);
 		cell_destroy(sp);
 		cellcpy(sp, cp);
@@ -338,7 +363,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	       has an ARRAY in the ptr field, replace expr
 	       with  & array[expr]
 	     */
-	    if (fp != 0) {
+	    if (fp != NULL) {
 		cp = array_find((ARRAY) fp[(cdp++)->op].ptr, sp, CREATE);
 		cell_destroy(sp);
 		sp->ptr = (PTR) cp;
@@ -350,7 +375,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	       has an ARRAY in the ptr field. Push this ARRAY
 	       on the eval stack
 	     */
-	    if (fp != 0) {
+	    if (fp != NULL) {
 		inc_sp();
 		sp->ptr = fp[(cdp++)->op].ptr;
 	    }
@@ -371,7 +396,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 		    cdp[0].op = A_PUSHA;
 		    cdp[1].ptr = stp->stval.array;
 		    assert(cdp[2].op == _BUILTIN);
-		    cdp[3].ptr = (PTR) bi_alength;
+		    cdp[3].fnc = bi_alength;
 		    break;
 		case ST_NONE:
 		    cdp[0].op = _PUSHI;
@@ -405,7 +430,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 		    cdp[0].op = LA_PUSHA;
 		    cdp[1].op = offset;
 		    assert(cdp[2].op == _BUILTIN);
-		    cdp[3].ptr = (PTR) bi_alength;
+		    cdp[3].fnc = bi_alength;
 		    break;
 		case ST_LOCAL_NONE:
 		    cdp[0].op = _PUSHI;
@@ -439,7 +464,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	case ALOOP:
 	    {
 		ALOOP_STATE *ap = aloop_state;
-		if (ap != 0 && (ap->ptr < ap->limit)) {
+		if (ap != NULL && (ap->ptr < ap->limit)) {
 		    cell_destroy(ap->var);
 		    ap->var->type = C_STRING;
 		    ap->var->ptr = (PTR) * ap->ptr++;
@@ -454,7 +479,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	    {
 		/* finish up an array loop */
 		ALOOP_STATE *ap = aloop_state;
-		if (ap != 0) {
+		if (ap != NULL) {
 		    aloop_state = ap->link;
 		    while (ap->ptr < ap->limit) {
 			free_STRING(*ap->ptr);
@@ -884,7 +909,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 
 	case _BUILTIN:
 	case _PRINT:
-	    sp = (*(PF_CP) (cdp++)->ptr) (sp);
+	    sp = (*(cdp++)->fnc) (sp);
 	    break;
 
 	case _POST_INC:
@@ -1145,21 +1170,21 @@ execute(INST * cdp,		/* code ptr, start execution here */
 
 	    if (begin_start) {
 		free_codes("BEGIN", begin_start, begin_size);
-		begin_start = 0;
+		begin_start = NULL;
 		begin_size = 0;
 	    }
 	    if (main_start) {
 		free_codes("MAIN", main_start, main_size);
-		main_start = 0;
+		main_start = NULL;
 		main_size = 0;
 	    }
-	    sp = eval_stack - 1;	/* might be in user function */
+	    sp = stack_under;	/* might be in user function */
 	    CLEAR_ALOOP_STACK();	/* ditto */
 	    break;
 
 	case _JMAIN:		/* go from BEGIN code to MAIN code */
 	    free_codes("BEGIN", begin_start, begin_size);
-	    begin_start = 0;
+	    begin_start = NULL;
 	    begin_size = 0;
 	    cdp = main_start;
 	    break;
@@ -1314,7 +1339,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 		int a_args = (cdp++)->op;	/* actual number of args */
 		CELL *nfp = sp - a_args + 1;	/* new fp for callee */
 		CELL *local_p = sp + 1;		/* first local argument on stack */
-		SYM_TYPE *type_p = 0;	/* pts to type of an argument */
+		SYM_TYPE *type_p = NULL;	/* pts to type of an argument */
 
 		if (fbp->nargs) {
 		    type_p = fbp->typev + a_args - 1;
@@ -1337,7 +1362,7 @@ execute(INST * cdp,		/* code ptr, start execution here */
 
 		/* cleanup the callee's arguments */
 		/* putting return value at top of eval stack */
-		if ((type_p != 0) && (sp >= nfp)) {
+		if ((type_p != NULL) && (sp >= nfp)) {
 		    cp = sp + 1;	/* cp -> the function return */
 
 		    do {
@@ -1464,7 +1489,6 @@ compare(CELL *cp)
 
     default:			/* there are no default cases */
 	bozo("bad cell type passed to compare");
-	break;
     }
     return result;
 }
@@ -1506,7 +1530,6 @@ cellcpy(CELL *target, CELL *source)
 
     default:
 	bozo("bad cell passed to cellcpy()");
-	break;
     }
     return target;
 }
