@@ -1,6 +1,6 @@
 /********************************************
 init.c
-copyright 2008-2021,2023, Thomas E. Dickey
+copyright 2008-2023,2024, Thomas E. Dickey
 copyright 1991-1994,1995, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,10 +11,16 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: init.c,v 1.77 2023/11/01 07:54:27 tom Exp $
+ * $MawkId: init.c,v 1.85 2024/12/14 21:21:20 tom Exp $
  */
 
-/* init.c */
+#define Visible_ARRAY
+#define Visible_BI_REC
+#define Visible_CELL
+#define Visible_PFILE
+#define Visible_STRING
+#define Visible_SYMTAB
+
 #include <mawk.h>
 #include <code.h>
 #include <memory.h>
@@ -102,13 +108,13 @@ static const struct {
 };
 /* *INDENT-ON* */
 
-static void
+static GCC_NORETURN void
 no_program(void)
 {
     mawk_exit(0);
 }
 
-static void
+static GCC_NORETURN void
 usage(FILE *fp)
 {
     static const char msg[][80] =
@@ -134,7 +140,6 @@ usage(FILE *fp)
 #if USE_BINMODE
 	"    -W binmode",
 #endif
-	"    -W compat        pre-POSIX 2001.",
 	"    -W dump          show assembler-like listing of program and exit.",
 	"    -W help          show this message and exit.",
 	"    -W interactive   set unbuffered output, line-buffered input.",
@@ -142,6 +147,7 @@ usage(FILE *fp)
 	"    -W posix         stricter POSIX checking.",
 	"    -W random=number set initial random seed.",
 	"    -W sprintf=number adjust size of sprintf buffer.",
+	"    -W traditional   pre-POSIX 2001.",
 	"    -W usage         show this message and exit.",
     };
     size_t n;
@@ -164,7 +170,7 @@ ok_abbrev(const char *fullName, const char *partName, int partLen)
 	UChar ch = (UChar) partName[n];
 	if (isalpha(ch))
 	    ch = (UChar) toupper(ch);
-	if (ch != (UChar) toupper(fullName[n])) {
+	if (ch != (UChar) toupper((UChar) fullName[n])) {
 	    result = 0;
 	    break;
 	}
@@ -187,7 +193,7 @@ haveValue(char *value)
     int result = 0;
 
     if (*value++ == '=') {
-	if (*value != '\0' && strchr("=,", *value) == 0)
+	if (*value != '\0' && strchr("=,", *value) == NULL)
 	    result = 1;
     }
     return result;
@@ -349,7 +355,8 @@ handle_w_opt(W_OPTIONS code, int glue, char *option, char **value)
     switch (code) {
     case W_VERSION:
 	print_version(stdout);
-	break;
+	/* NOTREACHED */
+
 #if USE_BINMODE
     case W_BINMODE:
 	wantArg = 1;
@@ -375,7 +382,7 @@ handle_w_opt(W_OPTIONS code, int glue, char *option, char **value)
 	    mawk_exit(2);
 	}
 	wantArg = 1;
-	if (optNext != 0) {
+	if (optNext != NULL) {
 	    pfile_name = optNext;
 	    wantArg = 2;
 	} else {
@@ -395,7 +402,7 @@ handle_w_opt(W_OPTIONS code, int glue, char *option, char **value)
 
     case W_RANDOM:
 	wantArg = 1;
-	if (optNext != 0) {
+	if (optNext != NULL) {
 	    long x = numeric_option(optNext);
 	    CELL c[2];
 
@@ -416,12 +423,12 @@ handle_w_opt(W_OPTIONS code, int glue, char *option, char **value)
 
     case W_SPRINTF:
 	wantArg = 1;
-	if (optNext != 0) {
+	if (optNext != NULL) {
 	    long x = numeric_option(optNext);
 
 	    if (x > (long) sizeof(string_buff)) {
 		if (sprintf_buff != string_buff &&
-		    sprintf_buff != 0) {
+		    sprintf_buff != NULL) {
 		    zfree(sprintf_buff,
 			  (size_t) (sprintf_limit - sprintf_buff));
 		}
@@ -437,7 +444,7 @@ handle_w_opt(W_OPTIONS code, int glue, char *option, char **value)
     case W_USAGE:
 	usage(stdout);
 	/* NOTREACHED */
-	break;
+
     case W_UNKNOWN:
 	errmsg(0, "vacuous option: -W \"%s\"", option);
 	break;
@@ -466,7 +473,7 @@ handle_w_opt(W_OPTIONS code, int glue, char *option, char **value)
     return result;
 }
 
-static void
+static GCC_NORETURN void
 bad_option(const char *s)
 {
     errmsg(0, "not an option: %s", s);
@@ -479,12 +486,12 @@ allow_long_options(char *arg, W_OPTIONS seen)
     char *env = getenv("MAWK_LONG_OPTIONS");
     int result = 0;
 
-    if (env != 0) {
+    if (env != NULL) {
 	switch (*env) {
 	default:
 	case 'e':		/* error */
 	    bad_option(arg);
-	    break;
+	    /* NOTREACHED */
 	case 'w':		/* warn */
 	    errmsg(0, "ignored option: %s", arg);
 	    break;
@@ -576,6 +583,7 @@ process_cmdline(int argc, char **argv)
 		optNext = optValue;
 	    } else {
 		bad_option(curArg);
+		/* NOTREACHED */
 	    }
 	    continue;
 	}
@@ -587,6 +595,7 @@ process_cmdline(int argc, char **argv)
 		    mawk_exit(2);
 		}
 		bad_option(curArg);
+		/* NOTREACHED */
 	    }
 
 	    optArg = argv[i + 1];
@@ -648,6 +657,7 @@ process_cmdline(int argc, char **argv)
 	case '-':
 	    if (curArg[2] != 0) {
 		bad_option(curArg);
+		/* NOTREACHED */
 	    }
 	    curArg = argv[++i];
 	    goto no_more_opts;
@@ -665,6 +675,7 @@ process_cmdline(int argc, char **argv)
 
 	default:
 	    bad_option(curArg);
+	    /* NOTREACHED */
 	}
     }
 
@@ -776,7 +787,7 @@ static ALL_ARRAYS *all_arrays;
 void
 array_leaks(void)
 {
-    while (all_arrays != 0) {
+    while (all_arrays != NULL) {
 	ALL_ARRAYS *next = all_arrays->next;
 	array_clear(all_arrays->a);
 	ZFREE(all_arrays->a);
