@@ -1,6 +1,6 @@
 /********************************************
 regexp.h
-copyright 2009-2014,2020, Thomas E. Dickey
+copyright 2009-2020,2024, Thomas E. Dickey
 copyright 2005, Aleksey Cheusov
 copyright 1991,1993, Michael D. Brennan
 
@@ -12,24 +12,23 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: regexp.h,v 1.15 2020/10/16 22:43:29 tom Exp $
+ * $MawkId: regexp.h,v 1.17 2024/08/25 17:15:43 tom Exp $
  */
 #ifndef  MAWK_REPL_H
 #define  MAWK_REPL_H
 
 #include <stdio.h>
-#include "nstd.h"
+#include <nstd.h>
 
 const char *REerror(void);
 
 #ifdef LOCAL_REGEXP
-#include "rexp.h"
+#include <rexp.h>
 STATE *REcompile(char *, size_t);
 void REdestroy(STATE *);
 int REtest(char *, size_t, STATE *);
 char *REmatch(char *, size_t, STATE *, size_t *, int);
 void REmprint(STATE *, FILE *);
-const char *REs_type(STATE *);
 #else
 PTR REcompile(char *, size_t);
 void REdestroy(PTR);
