@@ -1,6 +1,6 @@
 /********************************************
 repl.h
-copyright 2009-2020,2023, Thomas E. Dickey
+copyright 2009-2023,2024, Thomas E. Dickey
 copyright 1991,1993, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: repl.h,v 1.11 2023/07/20 00:32:26 tom Exp $
+ * $MawkId: repl.h,v 1.14 2024/09/05 17:24:56 tom Exp $
  */
 
 /* repl.h */
@@ -19,13 +19,27 @@ the GNU General Public License, version 2, 1991.
 #ifndef  REPL_H
 #define  REPL_H
 
-#include "types.h"
+#include <types.h>
 
-typedef struct re_data {
+typedef struct _re_data
+#ifdef Visible_RE_DATA
+{
     PTR compiled;		/* must be first... */
     int anchored;		/* use to limit recursion in gsub */
     int is_empty;		/* check if pattern is empty */
-} RE_DATA;
+}
+#endif
+RE_DATA;
+
+typedef struct _re_node
+#ifdef Visible_RE_NODE
+{
+    RE_DATA re;			/* keep this first, for re_destroy() */
+    STRING *sval;
+    struct _re_node *link;
+}
+#endif
+RE_NODE;
 
 /*
  * re_compile returns a RE_DATA*, but mawk handles it as a PTR thereafter.
@@ -33,9 +47,8 @@ typedef struct re_data {
 #define isAnchored(ptr) (((RE_DATA *)(ptr))->anchored)
 #define isEmpty_RE(ptr) (((RE_DATA *)(ptr))->is_empty)
 #define cast_to_re(ptr) (((RE_DATA *)(ptr))->compiled)
-#define refRE_DATA(re)  ((PTR) &(re))
 
-PTR re_compile(STRING *);
+RE_NODE *re_compile(STRING *);
 STRING *re_uncompile(PTR);
 
 CELL *repl_compile(STRING *);
